package de.opalpiraten.scenepainter.tools.basic;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.event.*;
import de.opalpiraten.scenepainter.gui.*;
import de.opalpiraten.scenepainter.sys.*;
import de.opalpiraten.scenepainter.gui.render.*;
import com.threerings.media.image.*;
import com.samskivert.util.StringUtil;

/**
 * This tool gives access to the Colorpository, allowing to modify
 * colors
 */
public class Coloring extends JInternalFrame implements ActionListener,
WorkspaceListener, InternalFrameListener {
  
  /**
   * Stopper for the listener
   */
  private boolean noupdate=false;
  
  /**
   * List of the colorclass names
   */
  private JComboBox classlist = new JComboBox();
  
  /**
   * List of the colors in the selected colorclass
   */
  private JComboBox colorlist = new JComboBox();
  
  /**
   * Clear private colorpository from project
   */
  private JButton clear = new JButton(Dict.get("tools.basic.coloring.clear"));
  
  /**
   * Info about the colorclass
   */
  private JTextArea classinfo = new JTextArea(5,25);
  
  /**
   * Info about a color
   */
  private JTextArea colorinfo = new JTextArea(5,25);
  
  /**
   * Project to use for loading/saving
   */
  private Project project;
  
  /**
   * Color preview
   */
  private JPanel showcolor = new JPanel();
  
  /**
   * Color preview
   */
  private JPanel showclass = new JPanel();
  
  /**
   * For submitting a colorname
   */
  private JTextField colorname = new JTextField(20);
  
  /**
   * For submitting a target color
   */
  private JTextField colortarget = new JTextField(10);
  
  public Coloring() {
    super(Dict.get("tools.basic.coloring.title",""),true,true,true,true);
    JPanel content = new JPanel();
    showcolor.add(new JLabel(" "));
    showclass.add(new JLabel(" "));
    
    GridBagLayout gbl = new GridBagLayout();
    content.setLayout(gbl);
    
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=1.0;
    gbc.weighty=1.0;
    gbc.insets = new Insets(1,1,1,1);
    JPanel classinfopanel = new JPanel();
    classinfopanel.add(classlist);
    classinfopanel.add(showclass);
    classinfopanel.add(new JScrollPane(classinfo));
    classinfopanel.setLayout(new BoxLayout(classinfopanel,BoxLayout.Y_AXIS));
    classinfopanel.setBorder(BorderFactory.createTitledBorder(Dict.get("tools.basic.coloring.classinfo.title")));
    gbl.setConstraints(classinfopanel,gbc);
    content.add(classinfopanel);
    

    gbc = new GridBagConstraints();
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=1;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=1.0;
    gbc.weighty=1.0;
    gbc.insets = new Insets(1,1,1,1);
    JPanel colorinfopanel = new JPanel();
    colorinfopanel.add(colorlist);
    colorinfopanel.add(showcolor);
    colorinfopanel.add(new JScrollPane(colorinfo));
    colorinfopanel.setLayout(new BoxLayout(colorinfopanel,BoxLayout.Y_AXIS));
    colorinfopanel.setBorder(BorderFactory.createTitledBorder(Dict.get("tools.basic.coloring.colorinfo.title")));
    gbl.setConstraints(colorinfopanel,gbc);
    content.add(colorinfopanel);
            
    
    gbc = new GridBagConstraints();
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=1;
    gbc.gridwidth=2; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    JPanel addmod = new JPanel();
    addmod.setBorder(BorderFactory.createTitledBorder(Dict.get("tools.basic.coloring.addmod.title")));
    addmod.add(colorname);
    addmod.add(colortarget);
    gbl.setConstraints(addmod,gbc);
    content.add(addmod);

    colorlist.addActionListener(this);
    classlist.addActionListener(this);
    colorname.addActionListener(this);
    colortarget.addActionListener(this);
    colorinfo.setEditable(false);
    classinfo.setEditable(false);
    
    colorname.setText("Bleen!");
    colortarget.setText("#000000");

    setContentPane(content);
    pack();
    setLocation(Core.workspace.getPlacementHint(this));
    Core.workspace.addWorkspaceListener(this);
    replaceProject(Core.workspace.getProject());
  }
  
  public void replaceProject(Project p) {
    noupdate=true;
    project = p;
    ColorPository cpos = p.getColorPository();
    Iterator it = cpos.enumerateClasses();
    Vector tmp = new Vector();
    while(it.hasNext()) tmp.add(((ColorPository.ClassRecord)it.next()).name);
    String[] classnames= new String[tmp.size()];
    tmp.copyInto(classnames);
    Arrays.sort(classnames);
    
    DefaultComboBoxModel cbm = new DefaultComboBoxModel(classnames);
    classlist.setModel(cbm);
    handleClassList();
    noupdate=false;
  }
  
  public void replaceScene(Scene s) {}
  
  
  private void handleClassList() {
    ColorPository cpos = project.getColorPository();
    ColorPository.ClassRecord crec = cpos.getClassRecord(classlist.getSelectedItem().toString());
    ColorPository.ColorRecord[] rec = cpos.enumerateColors(crec.name);
    String names[] = new String[rec.length];
    for (int i=0;i<names.length;i++) names[i] = rec[i].name;
    Arrays.sort(names);
    DefaultComboBoxModel cbm = new DefaultComboBoxModel(names);
    colorlist.setModel(cbm);

    
    Object[] param = {
      new Integer(crec.classId),
      Util.toHexString(crec.source),
      StringUtil.toString(crec.range)
    };
    
    classinfo.setText(Dict.get("tools.basic.coloring.classinfo.text",param));
    handleColorList();
    showclass.setBackground(crec.source);
  }
  
  private void handleColorList() {
    ColorPository cpos = project.getColorPository();
    ColorPository.ColorRecord rec = cpos.getColorRecord(classlist.getSelectedItem().toString(),colorlist.getSelectedItem().toString());
    Object[] param = {
      new Integer(rec.colorId),
      Util.toHexString(rec.getColorization().getColorizedRoot()),
      StringUtil.toString(rec.offsets)
    };
    colorinfo.setText(Dict.get("tools.basic.coloring.colorinfo.text",param));
    showcolor.setBackground(rec.getColorization().getColorizedRoot());
  }
  
  private void handleAddColor() {
    if (colorname.getText().equals("") || colortarget.getText().equals("")) return;
    ColorPository cpos = project.getColorPository();
    ColorPository.ClassRecord crec = cpos.getClassRecord(classlist.getSelectedItem().toString());
    
    try {
      String name = colorname.getText();
      float off[] = calcOffsets(crec.source,Color.decode(colortarget.getText()));

      ColorPository.ColorRecord col = cpos.getColorRecord(crec.name,name);
      if (col ==null) {
        int[] id = cpos.enumerateColorIds(crec.name);
        int newid=0;
        for (int i=0;i<id.length;i++) {
          if (id[i]>newid) newid=id[i]+1;
        }
        col =new ColorPository.ColorRecord();
        col.name = colorname.getText();
        col.offsets=calcOffsets(crec.source,Color.decode(colortarget.getText()));
        crec.addColor(col);
      
        project.setColorPository(cpos);
        Core.nenyabrowser.getTileManager().clearCache();
        Core.workspace.setProject(project);
      }
    }
    catch (Exception exp) {
      exp.printStackTrace();
    }
  }
  
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    if (src==classlist) handleClassList();
    if (src==colorlist) handleColorList();
    if (src==colorname || src==colortarget) handleAddColor();
  }
  
  
  /**
   * Calculate difference between two colors
   */
  private float[] calcOffsets(Color from, Color to) {
    float in[]=Color.RGBtoHSB(from.getRed(),from.getGreen(),from.getBlue(),null);
    float out[]=Color.RGBtoHSB(to.getRed(),to.getGreen(),to.getBlue(),null);
    float off[] = new float[3];
    off[0]=-in[0]+out[0];
    off[1]=-in[1]+out[1];
    off[2]=-in[2]+out[2];
    return off;
  }
  
  public void internalFrameActivated(InternalFrameEvent e) {
  }
  
  public void internalFrameClosed(InternalFrameEvent e) {}
  public void internalFrameClosing(InternalFrameEvent e) {
    Core.workspace.removeWorkspaceListener(this);
  }

  public void internalFrameDeactivated(InternalFrameEvent e) {}
  public void internalFrameDeiconified(InternalFrameEvent e) {}
  public void internalFrameIconified(InternalFrameEvent e) {}
  public void internalFrameOpened(InternalFrameEvent e) {}

}
