package de.opalpiraten.scenepainter.tools.basic;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.util.regex.*;
import java.util.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.event.*;
import de.opalpiraten.scenepainter.gui.*;
import de.opalpiraten.scenepainter.gui.render.*;

import de.opalpiraten.scenepainter.sys.*;
import com.threerings.media.image.*;


/**
 * A Tool for picking colors (for informational purposes).
 */
public class Pipette extends JInternalFrame implements ActionListener,
InternalFrameListener, SceneListener, WorkspaceListener, MouseMotionListener {
  
  /**
   * The scene to work on
   */
  private Scene scene;
  
  /**
   * Shows the picked color as a hexstring
   */
  private JTextField colname = new JTextField(6);
  
  /**
   * Opens up a JColorChooser
   */
  private JButton chooser = new JButton(Dict.get("tools.basic.pipette.chooser"));
  
  /**
   * Shows the picked color
   */
  private JPanel selcol = new JPanel();
  
  /**
   * Shows a miniature of the buffered image for navigation purposes.
   */
  private JPanel mini = new JPanel(); 
  
  /**
   * Buffers the scene, used for colorpicking
   */
  private BufferedImage buffer;
  
  /**
   * If true, the color information will not update while the mouse
   * travels the scene
   */
  private boolean noupdate=false;
  
  /**
   * Position of the cursor
   */
  private JLabel pos = new JLabel();
  
  
  /**
   * Sole constructor
   */
  public Pipette() {
    super(Dict.get("tools.basic.pipette.title"),true,true,true,true);
    JPanel content = new JPanel();
    selcol.setPreferredSize(new Dimension(25,25));
    mini.setPreferredSize(new Dimension(64,64));
    mini.setMinimumSize(new Dimension(64,64));
    mini.setMaximumSize(new Dimension(64,64));
    
    GridBagLayout gbl = new GridBagLayout();
    content.setLayout(gbl);
    GridBagConstraints gbc = new GridBagConstraints();
    
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=3;
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(pos,gbc);
    content.add(pos);
    
    gbc.anchor=GridBagConstraints.CENTER;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=0;
    gbc.gridy=1;
    gbc.gridwidth=3;
    gbc.gridheight=1;
    gbc.weightx=1.0;
    gbc.weighty=1.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(mini,gbc);  
    content.add(mini);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=0;
    gbc.gridy=2;
    gbc.gridwidth=1;
    gbc.gridheight=1;
    gbc.weightx=0.4;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(selcol,gbc);
    content.add(selcol);
    
    gbc.anchor=GridBagConstraints.CENTER;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=1;
    gbc.gridy=2;
    gbc.gridwidth=1;
    gbc.gridheight=1;
    gbc.weightx=0.4;
    gbc.weighty=0.0;
    gbc.insets = new Insets(2,2,2,2);
    gbl.setConstraints(colname,gbc);
    content.add(colname);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=2;
    gbc.gridy=2;
    gbc.gridwidth=1;
    gbc.gridheight=1;
    gbc.weightx=0.4;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(chooser,gbc);
    content.add(chooser);
    
    setContentPane(content);
    
    Integer[] tmp = {new Integer(0), new Integer(0)};
    pos.setText(Dict.get("tools.basic.pipette.pos",tmp));
    
    replaceScene(Core.workspace.getScene());
    colname.addActionListener(this);
    chooser.addActionListener(this);
    selcol.setBackground(Color.BLACK);
    pack();
    setLocation(Core.workspace.getPlacementHint(this));
    addInternalFrameListener(this);
    Core.workspace.addWorkspaceListener(this);
  }
  
  public void sceneChanged(Scene s) {
    buffer = new BufferedImage(s.getWidth(),s.getHeight(),BufferedImage.TYPE_INT_ARGB);
    s.paint(buffer.createGraphics());
  }
  
  public void replaceScene(Scene s) {
    if (scene!=null) {
      scene.removeSceneListener(this);
      scene.removeMouseMotionListener(this);
    }

    scene=s;
    
    scene.addSceneListener(this);
    scene.addMouseMotionListener(this);
    
    scene.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
    buffer = new BufferedImage(s.getWidth(),s.getHeight(),BufferedImage.TYPE_INT_ARGB);
    s.paint(buffer.createGraphics());
    
  }
  
  public void mouseMoved(MouseEvent e) {
    if (e.isAltDown()) return;
    Point p = e.getPoint();
    Color c = Color.BLACK;
    try {
      c =new Color(buffer.getRGB(p.x,p.y));
    }
    catch (Exception exp) {
      // The GTK+ Look and Feel may trigger 
      // java.lang.ArrayIndexOutOfBoundsException: Coordinate out of bounds!
      //e.printStackTrace();
    }
    colname.setText(Util.toHexString(c));
    selcol.setBackground(c);
    Integer[] tmp = {new Integer(p.x), new Integer(p.y)};
    pos.setText(Dict.get("tools.basic.pipette.pos",tmp));
    
    Graphics minigr = mini.getGraphics();
    minigr.clearRect(0,0,64,64);
    minigr.drawImage(buffer,0,0,64,64,p.x-16,p.y-16,p.x+16,p.y+16,null);
    minigr.setColor(Color.BLACK);
    minigr.drawLine(32,0,32,64);
    minigr.drawLine(0,32,64,32);
  }
  
  public void mouseDragged(MouseEvent e) {}
  
  public void replaceProject(Project p) {}
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    if (src==colname) {
      try {
        selcol.setBackground(Color.decode(colname.getText()));
      }
      catch (Exception exp) {
        // Don't bother
      }
    }
    
    if (src==chooser) {
      try {
        Color c= Color.decode(colname.getText());
        c = JColorChooser.showDialog(Core.workspace,Dict.get("tools.basic.pipette.chooser.title"), c);
        selcol.setBackground(c);
        colname.setText(Util.toHexString(c));
      }
      catch (Exception exp) {
        selcol.setBackground(Color.BLACK);
        colname.setText(Util.toHexString(Color.BLACK));
      }
    }
  }
  
  
  public void internalFrameActivated(InternalFrameEvent e) {
  }
  
  public void internalFrameClosed(InternalFrameEvent e) {}
  public void internalFrameClosing(InternalFrameEvent e) {
    Core.workspace.removeWorkspaceListener(this);
    scene.removeSceneListener(this);
    scene.removeMouseMotionListener(this);
    scene.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
  }

  public void internalFrameDeactivated(InternalFrameEvent e) {}
  public void internalFrameDeiconified(InternalFrameEvent e) {}
  public void internalFrameIconified(InternalFrameEvent e) {}
  public void internalFrameOpened(InternalFrameEvent e) {}
 
}
